/*================================================================
  File:     1_QueryStore_In_Action.sql

  Summary:  Intro to Query Store

  SQL Server Versions: SQL Server 2016
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2016, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/
USE [AdventureWorks2016E];
GO

/*
	add some data to the table
*/
USE [AdventureWorks2016E];
GO

BEGIN TRANSACTION

BULK INSERT [AdventureWorks2016E].[Sales].[SalesOrderDetail]
FROM 'C:\SQLskills\SQLintersection\QueryStore\sod.txt'
	WITH
	(
	DATAFILETYPE = 'native',
	TABLOCK
	);
GO

EXEC [Sales].[usp_GetProductInfo] 870

COMMIT

/*
	find the regressed queries in the UI
	look at query in detail
	what caused the plan change?
*/
SELECT [ProductID], COUNT([ProductID])
FROM [Sales].[SalesOrderDetail]
GROUP BY [ProductID]
ORDER BY COUNT([ProductID]) DESC;
GO

/*
	what's in cache now?
*/
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
GO
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, cp.size_in_bytes/1024 [PlanSizeKB], qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
JOIN sys.dm_exec_cached_plans cp ON qs.plan_handle = cp.plan_handle
WHERE s.text like '%usp_GetProductInfo%'
OPTION (MAXDOP 1);
GO
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
GO

/*
	force the plan in the UI
*/
/*
	a plan can be forced in the UI
	or with a SP
*/
EXEC [sys].[sp_query_store_force_plan] @query_id = 6, @plan_id = 1561;


/*
	check status of forced plan
*/
SELECT p.plan_id, p.query_id, q.object_id as containing_object_id,
    force_failure_count, last_force_failure_reason_desc
FROM sys.query_store_plan AS p
JOIN sys.query_store_query AS q on p.query_id = q.query_id
WHERE is_forced_plan = 1;


/*
	what's in cache now?
*/
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
GO
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, cp.size_in_bytes/1024 [PlanSizeKB], qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
JOIN sys.dm_exec_cached_plans cp ON qs.plan_handle = cp.plan_handle
WHERE s.text like '%usp_GetProductInfo%'
OPTION (MAXDOP 1);
GO
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
GO

/*
	What happens when I run the query?
	(enable plan)
*/
EXEC [Sales].[usp_GetProductInfo] 919;
GO

/*	
	stop all scripts
	what if I clear the cache?
*/
DBCC FREEPROCCACHE;
GO

/*
	run it with a big value
	(enable the actual plan)
*/
SELECT [ProductID], COUNT(*)
FROM [Sales].[SalesOrderDetail]
GROUP BY [ProductID]
ORDER BY COUNT(*) DESC;
GO

EXEC [Sales].[usp_GetProductInfo] 711;
GO


SELECT [ProductID], [OrderQty]
FROM [Sales].[SalesOrderDetail]
WHERE [ProductID] = 711
ORDER BY [ProductID];

/*
	check cache again for the plan
*/
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
GO
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, cp.size_in_bytes/1024 [PlanSizeKB], qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
JOIN sys.dm_exec_cached_plans cp ON qs.plan_handle = cp.plan_handle
WHERE s.text like '%usp_GetProductInfo%'
OPTION (MAXDOP 1);
GO
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
GO


/*
	what does the estimated plan show?
*/
EXEC [Sales].[usp_GetProductInfo] 711;
GO


/*
	what if I add a RECOMPILE?
	(enable actual plan)
*/
EXEC [Sales].[usp_GetProductInfo] 711 WITH RECOMPILE;
GO
