/*================================================================
  File:     3_PlanGuidesANDForcing.sql

  Summary:  Intro to Query Store

  SQL Server Versions: SQL Server 2016
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2016, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/

USE [master];
GO
RESTORE DATABASE [WideWorldImporters] 
	FROM  DISK = N'C:\Backups\WideWorldImporters-Full.bak' 
	WITH  FILE = 1,  
	MOVE N'WWI_Primary' TO N'C:\Databases\WideWorldImporters\WideWorldImporters.mdf',  
	MOVE N'WWI_UserData' TO N'C:\Databases\WideWorldImporters\WideWorldImporters_UserData.ndf',  
	MOVE N'WWI_Log' TO N'C:\Databases\WideWorldImporters\WideWorldImporters.ldf',  
	MOVE N'WWI_InMemory_Data_1' TO N'C:\Databases\WideWorldImporters\WideWorldImporters_InMemory_Data_1',  
	NOUNLOAD, 
	REPLACE, 
	STATS = 5;
GO


/*
	Create a procedure for testing
*/
USE [WideWorldImporters];
GO

DROP PROCEDURE IF EXISTS [Sales].[usp_GetFullProductInfo];
GO

CREATE PROCEDURE [Sales].[usp_GetFullProductInfo]
	@StockItemID INT
AS	

	SELECT 
		[o].[CustomerID], 
		[o].[OrderDate], 
		[ol].[StockItemID], 
		[ol].[Quantity],
		[ol].[UnitPrice]
	FROM [Sales].[Orders] [o]
	JOIN [Sales].[OrderLines] [ol] on [o].[OrderID] = [ol].[OrderID]
	WHERE [StockItemID] = @StockItemID
	ORDER BY [o].[OrderDate] DESC;

GO

/*
	Enable actual plan and STATISTICS IO
	Execute the stored procecdure
*/
SET STATISTICS IO ON;
GO
EXEC [Sales].[usp_GetFullProductInfo] 220 WITH RECOMPILE;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130 WITH RECOMPILE;
GO

SET STATISTICS IO OFF;
GO


/*
	deteremine which plan is "better" to force  

*/
EXEC sp_create_plan_guide   
    @name =  N'ProductInfoPlanGuide',  
    @stmt = N'SELECT 
		[o].[CustomerID], 
		[o].[OrderDate], 
		[ol].[StockItemID], 
		[ol].[Quantity],
		[ol].[UnitPrice]
		FROM [Sales].[Orders] [o]
		JOIN [Sales].[OrderLines] [ol] on [o].[OrderID] = [ol].[OrderID]
		WHERE [StockItemID] = @StockItemID
		ORDER BY [o].[OrderDate] DESC;',  
    @type = N'OBJECT',  
    @module_or_batch = N'[Sales].[usp_GetFullProductInfo]',  
    @params = NULL,  
    @hints = N'OPTION (OPTIMIZE FOR (@StockItemID = 220))'; 


/*
	verify plan is being used
*/
SET STATISTICS IO ON;
GO
EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

SET STATISTICS IO OFF;
GO

/*
	add some data
*/
INSERT INTO Sales.OrderLines
	(OrderLineID, OrderID, StockItemID, 
	Description, PackageTypeID, Quantity, 
	UnitPrice, TaxRate, PickedQuantity, 
	PickingCompletedWhen, LastEditedBy, 
	LastEditedWhen)
SELECT 
	OrderLineID + 500000, OrderID, StockItemID, 
	Description, PackageTypeID, Quantity + 2, 
	UnitPrice, TaxRate, PickedQuantity, 
	NULL, LastEditedBy, SYSDATETIME() 
FROM WideWorldImporters.Sales.OrderLines 
WHERE OrderID > 54999;
GO

/*
	check the plans again
*/
SET STATISTICS IO ON;
GO
EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

SET STATISTICS IO OFF;
GO

/*
	check to see what the system tells us
*/
SELECT *
FROM sys.plan_guides;
GO

SELECT *
FROM sys.fn_validate_plan_guide (65536);
GO


/*
	reset and try with Query Store plan forcing
*/
USE [master];
GO
RESTORE DATABASE [WideWorldImporters] 
	FROM  DISK = N'C:\Backups\WideWorldImporters-Full.bak' 
	WITH  FILE = 1,  
	MOVE N'WWI_Primary' TO N'C:\Databases\WideWorldImporters\WideWorldImporters.mdf',  
	MOVE N'WWI_UserData' TO N'C:\Databases\WideWorldImporters\WideWorldImporters_UserData.ndf',  
	MOVE N'WWI_Log' TO N'C:\Databases\WideWorldImporters\WideWorldImporters.ldf',  
	MOVE N'WWI_InMemory_Data_1' TO N'C:\Databases\WideWorldImporters\WideWorldImporters_InMemory_Data_1',  
	NOUNLOAD, 
	REPLACE, 
	STATS = 5;
GO

/*
	Verify QS enabled
*/
USE [WideWorldImporters];
GO
SELECT actual_state_desc, readonly_reason, desired_state_desc, current_storage_size_mb, 
    max_storage_size_mb, flush_interval_seconds, interval_length_minutes, 
    stale_query_threshold_days, size_based_cleanup_mode_desc, 
    query_capture_mode_desc, max_plans_per_query
FROM sys.database_query_store_options;
GO



/*
	clear what's in the query store
*/
USE [master];
GO
ALTER DATABASE [WideWorldImporters] 
	SET QUERY_STORE CLEAR;
GO

/*
	Create a procedure for testing
*/
USE [WideWorldImporters];
GO

DROP PROCEDURE IF EXISTS [Sales].[usp_GetFullProductInfo];
GO

CREATE PROCEDURE [Sales].[usp_GetFullProductInfo]
	@StockItemID INT
AS	

	SELECT 
		[o].[CustomerID], 
		[o].[OrderDate], 
		[ol].[StockItemID], 
		[ol].[Quantity],
		[ol].[UnitPrice]
	FROM [Sales].[Orders] [o]
	JOIN [Sales].[OrderLines] [ol] on [o].[OrderID] = [ol].[OrderID]
	WHERE [StockItemID] = @StockItemID
	ORDER BY [o].[OrderDate] DESC;

GO

/*
	Enable actual plan and STATISTICS IO
	Execute the stored procecdure
*/
SET STATISTICS IO ON;
GO
EXEC [Sales].[usp_GetFullProductInfo] 220 WITH RECOMPILE;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130 WITH RECOMPILE;
GO

SET STATISTICS IO OFF;
GO


EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

EXEC sp_query_store_flush_db;
GO

DBCC FREEPROCCACHE;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

EXEC sp_query_store_flush_db;
GO

SELECT
	qsq.query_id, qsp.plan_id, qsq.object_id, 
	qsq.count_compiles, 
	LocalLastExecutionTime = DATEADD(MINUTE, -(DATEDIFF(MINUTE, GETDATE(), GETUTCDATE())), qsp.last_execution_time),
	qst.query_sql_text, 
	qsq.query_hash,
	ConvertedPlan = TRY_CONVERT(XML, qsp.query_plan)
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
JOIN (
	SELECT query_id AS query_id, COUNT(query_id) AS QueryCount
	FROM sys.query_store_plan
	GROUP BY query_id
	HAVING COUNT(query_id)> 1
	) m on qsq.query_id = m.query_id
WHERE qst.query_sql_text like '%@StockItemID%';
GO


/*
	Within QS, force the plan
	sp_query_store_force_plan (query_id, plan_id)
*/
EXEC sp_query_store_force_plan 4,2;
GO

EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

EXEC [Sales].[usp_GetFullProductInfo] 189;
GO

EXEC [Sales].[usp_GetFullProductInfo] 103;
GO

EXEC [Sales].[usp_GetFullProductInfo] 217;
GO



/*
	check status of forced plan
*/
SELECT 
	[p].[plan_id], 
	[p].[query_id], 
	[q].[object_id] AS containing_object_id,
    [p].[force_failure_count], 
	[p].[last_force_failure_reason_desc]
FROM [sys].[query_store_plan] AS [p]
JOIN [sys].[query_store_query] AS [q] on [p].[query_id] = [q].[query_id]
WHERE [is_forced_plan] = 1;
GO


/*
	add some data like we did before 
*/
INSERT INTO Sales.OrderLines
	(OrderLineID, OrderID, StockItemID, 
	Description, PackageTypeID, Quantity, 
	UnitPrice, TaxRate, PickedQuantity, 
	PickingCompletedWhen, LastEditedBy, 
	LastEditedWhen)
SELECT 
	OrderLineID + 500000, OrderID, StockItemID, 
	Description, PackageTypeID, Quantity + 2, 
	UnitPrice, TaxRate, PickedQuantity, 
	NULL, LastEditedBy, SYSDATETIME() 
FROM WideWorldImporters.Sales.OrderLines 
WHERE OrderID > 54999;
GO

/*
	does the same plan get used?
*/
EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

EXEC [Sales].[usp_GetFullProductInfo] 189;
GO

EXEC [Sales].[usp_GetFullProductInfo] 103;
GO

EXEC [Sales].[usp_GetFullProductInfo] 217;
GO



/*
	what if we add an index?
*/
CREATE NONCLUSTERED INDEX [NCI_OrderLines_StockItemID]
ON [Sales].[OrderLines] ([StockItemID])
INCLUDE ([OrderID],[Quantity],[UnitPrice]);
GO

/*
	re-run the SP to see if index used
*/
EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

/*
	un-force the plan and re-run
*/
EXEC sp_query_store_unforce_plan 4,2

/*
	check status of forced plan
*/
SELECT 
	[p].[plan_id], 
	[p].[query_id], 
	[q].[object_id] AS containing_object_id,
    [p].[force_failure_count], 
	[p].[last_force_failure_reason_desc]
FROM [sys].[query_store_plan] AS [p]
JOIN [sys].[query_store_query] AS [q] on [p].[query_id] = [q].[query_id]
WHERE [is_forced_plan] = 1;
GO


/*
	free cache, re-run SP
	which one uses our index?
*/
DBCC FREEPROCCACHE;
GO

EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

DBCC FREEPROCCACHE;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

/*
	Now what do we have in QS for plans?
*/
SELECT
	qsq.query_id, qsp.plan_id, qsq.object_id, 
	qsq.count_compiles, 
	LocalLastExecutionTime = DATEADD(MINUTE, -(DATEDIFF(MINUTE, GETDATE(), GETUTCDATE())), qsp.last_execution_time),
	qst.query_sql_text, 
	qsq.query_hash,
	ConvertedPlan = TRY_CONVERT(XML, qsp.query_plan)
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
JOIN (
	SELECT query_id AS query_id, COUNT(query_id) AS QueryCount
	FROM sys.query_store_plan
	GROUP BY query_id
	HAVING COUNT(query_id)> 1
	) m on qsq.query_id = m.query_id
WHERE qst.query_sql_text like '%@StockItemID%';
GO

/*
	Within QS, force the plan that uses the NCI
*/
EXEC sp_query_store_force_plan 4,7;
GO

/*
	check status of forced plan
*/
SELECT 
	[p].[plan_id], 
	[p].[query_id], 
	[q].[object_id] AS containing_object_id,
    [p].[force_failure_count], 
	[p].[last_force_failure_reason_desc]
FROM [sys].[query_store_plan] AS [p]
JOIN [sys].[query_store_query] AS [q] on [p].[query_id] = [q].[query_id]
WHERE [is_forced_plan] = 1;
GO

/*
	Test with SP
*/
EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

EXEC [Sales].[usp_GetFullProductInfo] 189;
GO

EXEC [Sales].[usp_GetFullProductInfo] 103;
GO

EXEC [Sales].[usp_GetFullProductInfo] 217;
GO

/*
	what if we drop the index?
*/
DROP INDEX [NCI_OrderLines_StockItemID] ON [Sales].[OrderLines]
GO


/*
	re-run the SP
*/
EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

EXEC [Sales].[usp_GetFullProductInfo] 189;
GO

EXEC [Sales].[usp_GetFullProductInfo] 103;
GO

EXEC [Sales].[usp_GetFullProductInfo] 217;
GO

/*
	check status of forced plan
*/
SELECT 
	[p].[plan_id], 
	[p].[query_id], 
	[q].[object_id] AS containing_object_id,
    [p].[force_failure_count], 
	[p].[last_force_failure_reason_desc]
FROM [sys].[query_store_plan] AS [p]
JOIN [sys].[query_store_query] AS [q] on [p].[query_id] = [q].[query_id]
WHERE [is_forced_plan] = 1;
GO

 

 /*
	recreate the index...but add another column
*/
CREATE NONCLUSTERED INDEX [NCI_OrderLines_StockItemID]
ON [Sales].[OrderLines] ([StockItemID])
INCLUDE ([OrderID],[Quantity],[UnitPrice], [TaxRate]);
GO


/*
	re-run the SP
*/
EXEC [Sales].[usp_GetFullProductInfo] 220;
GO

EXEC [Sales].[usp_GetFullProductInfo] 130;
GO

EXEC [Sales].[usp_GetFullProductInfo] 189;
GO

EXEC [Sales].[usp_GetFullProductInfo] 103;
GO

EXEC [Sales].[usp_GetFullProductInfo] 217;
GO

/*
	check status of forced plan
*/
SELECT 
	[p].[plan_id], 
	[p].[query_id], 
	[q].[object_id] AS containing_object_id,
    [p].[force_failure_count], 
	[p].[last_force_failure_reason_desc]
FROM [sys].[query_store_plan] AS [p]
JOIN [sys].[query_store_query] AS [q] on [p].[query_id] = [q].[query_id]
WHERE [is_forced_plan] = 1;
GO
